/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.jsp.tag;

import com.micronova.util.BeanUtil;
import com.micronova.util.TypeUtil;
import java.util.Collection;
import java.util.Iterator;
import javax.servlet.jsp.PageContext;

public class ELIterator implements Iterator
{
    protected PageContext _pageContext;
    protected Iterator _iterator;
    protected int _index;
    protected Collection _collection;
    protected Object _hasNext;
    protected Object _next;
    protected String _hasNextCodec;
    protected String _nextCodec;

    public ELIterator(PageContext pageContext, Object source) throws Exception
    {
        super();

        _pageContext = pageContext;
        _iterator = null;
        _index = 0;
        _collection = null;
        _hasNext = null;
        _next = null;
        _hasNextCodec = null;
        _nextCodec = null;

        BeanUtil.fill(this, TypeUtil.isNestedMap(source));
    }

    public void setIndex(int index)
    {
        _index = index;
    }

    public int getIndex()
    {
        return _index;
    }

    public void setCollection(Collection collection)
    {
        _collection = collection;
        _iterator = null;
    }

    public Collection getCollection()
    {
        return _collection;
    }

    public void setHasNextCodec(String hasNextCodec)
    {
        _hasNextCodec = hasNextCodec;
    }

    public String getHasNextCodec()
    {
        return _hasNextCodec;
    }

    public void setNextCodec(String nextCodec)
    {
        _nextCodec = nextCodec;
    }

    public String getNextCodec()
    {
        return _nextCodec;
    }

    public void setHasNext(Object hasNext)
    {
        _hasNext = hasNext;
    }

    public void setNext(Object next)
    {
        _next = next;
    }

    public boolean hasNext()
    {
        Object envVar = null;
        PageContext pageContext = _pageContext;

        try
        {
            envVar = YuzuTag.getValueVar(pageContext);

            YuzuTag.setValueVar(pageContext, this);

            String hasNextCodec = EL.replaceEvalEscape(_hasNextCodec);

            Object hasNextValue = null;

            if (!TypeUtil.isEmptyString(hasNextCodec))
            {
                hasNextValue = EL.applyCodec(pageContext, hasNextCodec, this);
            }
            else
            {
                hasNextValue = getHasNext();
            }

            return TypeUtil.isTrue(hasNextValue);
        }
        catch (Exception e)
        {
            e.printStackTrace();

            throw new RuntimeException(e);
        }
        finally
        {
            if (envVar != null)
            {
                YuzuTag.setValueVar(pageContext, envVar);
            }
        }
    }

    public Object next()
    {
        Object envVar = null;
        PageContext pageContext = _pageContext;

        try
        {
            envVar = YuzuTag.getValueVar(pageContext);

            YuzuTag.setValueVar(pageContext, this);

            String nextCodec = EL.replaceEvalEscape(_nextCodec);

            Object x = null;

            if (!TypeUtil.isEmptyString(nextCodec))
            {
                x = EL.applyCodec(pageContext, nextCodec, this);
            }
            else
            {
                x = getNext();
            }

            _index ++;

            return x;
        }
        catch (Exception e)
        {
            e.printStackTrace();

            throw new RuntimeException(e);
        }
        finally
        {
            if (envVar != null)
            {
                YuzuTag.setValueVar(pageContext, envVar);
            }
        }
    }

    public void remove()
    {
        throw new UnsupportedOperationException();
    }

    public Iterator getIterator()
    {
        Iterator iterator = _iterator;
        
        if (iterator == null)
        {
            Object collection = _collection;

            if (collection != null)
            {
                iterator = ((Collection)collection).iterator();
                _iterator = iterator;
            }
        }

        return iterator;
    }

    public Object getNext()
    {
        Object next = _next;

        if (next == null)
        {
            Iterator iterator = getIterator();

            if (iterator != null)
            {
                next = iterator.next();
            }
        }

        return next;
    }

    public Object getHasNext()
    {
        Object hasNext = _hasNext;

        if (hasNext != null)
        {
            if (TypeUtil.isTrue(hasNext))
            {
                hasNext = Boolean.TRUE;
            }
            else
            {
                hasNext = null;
            }
        }
        else
        {
            Iterator iterator = getIterator();

            if (iterator != null)
            {
                if (iterator.hasNext())
                {
                    hasNext = Boolean.TRUE;
                }
            }
        }
 
        return hasNext;
    }
}

