/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.jsp.tag;

import com.micronova.util.NestedMap;

/** NestedMap tag */

public class MapTag extends YuzuTag
{
    protected NestedMap _map;
    protected String _bodyProperty;
    protected Object _body;

    /** initialization */

    protected void init()
    {
        super.init();

        _bodyProperty = null;
    }

    protected void prepare() throws Exception
    {
        super.prepare();

        NestedMap map = _map;

        if (map == null)
        {
            map = new NestedMap();
            _map = map;
        }
        else
        {
            map.clear();
            map.setAcceptable(null);
        }

        _tagValue = map;

        _body = null;
    }

    /** resets map if assigned, otherwise clears */

    protected void cleanup()
    {
        if (_isAssigned)
        {
            _map = null;
        }
    }

    /** copy from source. */

    protected Object copyFromSource(Object tagValue, Object source) throws Exception
    {
        if (source != null)
        {
            ((NestedMap)tagValue).copyFromSource(source);
        }

        return tagValue;
    }

    protected boolean doesImport(Object tagValue)
    {
        return (!isEmptyString(_bodyProperty));
    }

    protected Object importBody(Object tagValue) throws Exception
    {
        if (_body == null)
        {
            Object importedValue = super.importBody(tagValue);

            if (importedValue != tagValue)
            {
                _body = importedValue;
            }
        }

        return tagValue;
    }

    protected Object processValue(Object tagValue) throws Exception
    {
        NestedMap map = _map;

        String bodyProperty = _bodyProperty;
        
        if (!isEmptyString(bodyProperty))
        {
            map.put(bodyProperty, _body);
        }

        return tagValue;
    }

    public void setBodyProperty(Object expression) throws Exception
    {
        _bodyProperty = (String)evaluateAttribute("bodyProperty", expression, String.class, _bodyProperty);
    }
}
