/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.jsp.tag;

import com.micronova.util.NestedMap;
import com.micronova.util.Shell;
import java.io.CharArrayReader;
import java.io.CharArrayWriter;
import java.io.Reader;
import java.io.Writer;

public class SystemTag extends MapTag
{
    public final static String COMMAND = "command";
    public final static String STDIN = "stdin";
    public final static String STDOUT = "stdout";
    public final static String STDERR = "stderr";
    public final static String RC = "rc";
    public final static String ENCODING = "encoding";

    private static final String DEFAULTEXPORT = "${" + VALUEVAR + "." + STDOUT + "}";

    protected String _command;
    protected Object _stdin;
    protected Object _stdout;
    protected Object _stderr;
    protected String _encoding;

    public void init()
    {
        super.init();

        _export = DEFAULTEXPORT;
        _command = null;
        _stdin = null;
        _stdout = null;
        _stderr = null;
        _encoding = "iso-8859-1";
    }

    protected void prepare() throws Exception
    {
        super.prepare();

        _map.put(COMMAND, _command);
        _map.put(STDIN, _stdin);
        _map.put(STDOUT, _stdout);
        _map.put(STDERR, _stderr);
        _map.put(ENCODING, _encoding);
    }

    public Object processValue(Object tagValue) throws Exception
    {
        NestedMap map = _map;

        String command = (String)map.get(COMMAND);
        Object stdin = map.get(STDIN);
        Object stdout = map.get(STDOUT);
        Object stderr = map.get(STDERR);
        String encoding = map.getString(ENCODING);

        if (stdin == null)
        {
            stdin = "";
        }

        if (command != null)
        {
            CharArrayReader cIn = null;
            CharArrayWriter cOut = null;
            CharArrayWriter cErr = null;

            Reader in = null;
            Writer out = null;
            Writer err = null;

            if (stdin instanceof Reader)
            {
                in = (Reader)stdin;
            }
            else
            {
                cIn = new CharArrayReader(stdin.toString().toCharArray());
                in = cIn;
            }

            if (stdout instanceof Writer)
            {
                out = (Writer)stdout;
            }
            else
            {
                cOut = new CharArrayWriter();
                out = cOut;
            }

            if (stderr instanceof Writer)
            {
                err = (Writer)stderr;
            }
            else
            {
                cErr = new CharArrayWriter();
                err = cErr;
            }

            Process p = Runtime.getRuntime().exec(command);
                
            Shell shell = new Shell(p, in, out, err, encoding);
                
            int rc = shell.exec();
            
            map.put(RC, new Integer(rc));

            out.flush();
            err.flush();
            
            if (cOut != null)
            {
                map.put(STDOUT, cOut.toString());
            }
            else
            {
                map.put(STDOUT, null);
            }
            
            if (cErr != null)
            {
                map.put(STDERR, cErr.toString());
            }
            else
            {
                map.put(STDERR, null);
            }
        }

        return tagValue;
    }

    public void setCommand(Object expression) throws Exception
    {
        _command = (String)evaluateAttribute(COMMAND, expression, String.class, _command);
    }

    public void setStdin(Object expression) throws Exception
    {
        _stdin = evaluateAttribute(STDIN, expression, Object.class, _stdin);
    }

    public void setStdout(Object expression) throws Exception
    {
        _stdout = evaluateAttribute(STDIN, expression, Object.class, _stdout);
    }

    public void setStderr(Object expression) throws Exception
    {
        _stderr = evaluateAttribute(STDIN, expression, Object.class, _stderr);
    }

}
