/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/


package com.micronova.jsp.tag;

import com.micronova.util.SQLObjectSource;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.List;
import java.util.regex.Pattern;
import javax.servlet.jsp.jstl.core.Config;
import org.apache.taglibs.standard.tag.common.sql.ResultImpl;

/** templated update statement **/

public class UpdateTag extends EvalTag
{
    protected Object _dataSource;
    protected List _nameList;
    protected StringBuffer _buffer;
    protected Integer _startRow;
    protected Integer _maxRows;

    protected void init()
    {
        super.init();

        _pattern = EL.DEFAULTPATTERNQUERY;
        _dataSource = null;
        _export = null;
        _startRow = new Integer(0);
        _maxRows = new Integer(-1);
    }

    protected void prepare() throws Exception
    {
        super.prepare();

        _nameList = null;

        if (_buffer == null)
        {
            _buffer = new StringBuffer();
        }
        else
        {
            _buffer.setLength(0);
        }
    }

    protected Object doSql(PreparedStatement preparedStatement, SQLObjectSource sqlObjectSource) throws Exception
    {
        if (sqlObjectSource.isCallable())
        {
            CallableStatement callableStatement = (CallableStatement)preparedStatement;

            boolean isResultSet = callableStatement.execute();
            
            if (isResultSet)
            {
                ResultSet resultSet = null;

                try
                {
                    resultSet = callableStatement.getResultSet();
                    
                    return new ResultImpl(resultSet, _startRow.intValue(), _maxRows.intValue());
                }
                catch (Exception e)
                {
                    throw e;
                }
                finally
                {
                    if (resultSet != null)
                    {
                        resultSet.close();
                    }
                }
            }
            else
            {
                return sqlObjectSource.getOutputs(callableStatement);
            }
        }
        else
        {
            return new Integer(preparedStatement.executeUpdate());
        }
    }

    protected Connection getConnection(Object spec) throws Exception
    {
        if (spec == null)
        {
            spec = getConfiguration(Config.SQL_DATA_SOURCE, null);
        }

        return TransactionTag.openConnection(spec);
    }

    protected Object evaluate(Object tagValue, Pattern pattern) throws Exception
    {
        Object value = null;
        String expression = tagValue.toString();

        Connection connection = null;
        boolean doesCloseConnection = false;
        PreparedStatement preparedStatement = null;

        try
        {
            Object dataSource = _dataSource;

            if (dataSource == null)
            {
                connection = TransactionTag.getAncestorConnection(pageContext, this);
            }

            if (connection == null)
            {
                doesCloseConnection = true;
               
                connection = getConnection(dataSource);
            }
            
            SQLObjectSource sqlObjectSource = new SQLObjectSource(this);

            preparedStatement = sqlObjectSource.render(connection, expression, pattern, _recursive.booleanValue(), _allowGroup.booleanValue());

            value = doSql(preparedStatement, sqlObjectSource);
        }
        catch (Exception e)
        {
            throw e;
        }
        finally
        {
            try
            {
                if (preparedStatement != null)
                {
                    preparedStatement.close();
                }
            }
            catch (Exception ee)
            {
                throw ee;
            }
            finally
            {
                if (doesCloseConnection)
                {
                    if (connection != null)
                    {
                        connection.close();
                    }
                }
            }
        }

        return value;
    }

    public void setDataSource(Object expression) throws Exception
    {
        Object dataSource = evaluateAttribute("dataSource", expression, Object.class, _dataSource);

        if (!isEmptyString(dataSource))
        {
            _dataSource = dataSource;
        }
        else
        {
            _dataSource = null;
        }
    }

    public void setStartRow(Object expression) throws Exception
    {
        _startRow = (Integer)evaluateAttribute("startRow", expression, Integer.class, _startRow);
    }

    public void setMaxRows(Object expression) throws Exception
    {
        _maxRows = (Integer)evaluateAttribute("maxRows", expression, Integer.class, _maxRows);
    }
}
