/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util;

import java.io.File;
import java.net.URI;
import java.net.URL;

/** Bean-ized wrapper for java.io.File */

public class FileBean
{
    File _file;

    public FileBean()
    {
        super();
    }

    public FileBean(File file)
    {
        this();
        setFileObject(file);
    }

    public FileBean(URI uri)
    {
        this();
        setUri(uri);
    }

    public void setFileObject(File file)
    {
        _file = file;
    }

    public File getFileObject()
    {
        return _file;
    }

    public boolean isCanRead()
    {
        return _file.canRead();
    }

    public boolean isCanWrite()
    {
        return _file.canWrite();
    }

    public boolean isExists()
    {
        return _file.exists();
    }

    public boolean isAbsolute()
    {
        return _file.isAbsolute();
    }
    
    public boolean isDirectory()
    {
        return _file.isDirectory();
    }

    public boolean isFile()
    {
        return _file.isFile();
    }

    public boolean isHidden()
    {
        return _file.isHidden();
    }

    public FileBean getAbsoluteFile()
    {
        return new FileBean(_file.getAbsoluteFile());
    }

    public String getAbsolutePath()
    {
        return _file.getAbsolutePath();
    }

    public FileBean getCanonicalFile() throws java.io.IOException
    {
        return new FileBean(_file.getCanonicalFile());
    }

    public String getCanonicalPath() throws java.io.IOException
    {
        return _file.getCanonicalPath();
    }

    public String getName()
    {
        return _file.getName();
    }

    public String getParent()
    {
        return _file.getParent();
    }

    public String getPath()
    {
        return _file.getPath();
    }

    public long getLastModified()
    {
        return _file.lastModified();
    }

    public long getLength()
    {
        return _file.length();
    }

    public URI getUri() throws java.net.MalformedURLException
    {
        return _file.toURI();
    }

    public void setUri(URI uri)
    {
        setFileObject(new File(uri));
    }

    public URL getUrl() throws java.net.MalformedURLException
    {
        return _file.toURL();
    }
}
