/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util;

import java.io.Writer;
import javax.servlet.ServletContext;
import javax.servlet.jsp.PageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/** Writer wrapper for logs */

public class LogWriter extends Writer
{
    PageContext _pageContext;
    String _destination;
    String _category;
    Log _log;

    public LogWriter()
    {
        super();
    }

    public LogWriter(PageContext pageContext, String destination, String category, Log log)
    {
        super();

        this.setPageContext(pageContext);
        this.setDestination(destination);
        this.setCategory(category);
        this.setLog(log);
    }

    public final void setPageContext(PageContext pageContext)
    {
        this._pageContext = pageContext;
    }

    public final void setDestination(String destination)
    {
        this._destination = destination;
    }

    public final void setCategory(String category)
    {
        this._category = category;
    }

    public final void setLog(Log log)
    {
        this._log = log;
    }

    public void close()
    {
        // no operation
    }

    public void flush()
    {
        // no operation
    }

    public void write(char[] cbuf, int off, int len)
    {
        String s = new String(cbuf, off, len);

        if ("log".equals(_destination))
        {
            ServletContext context = _pageContext.getServletContext();
            
            context.log(s);
        }
        else if ("out".equals(_destination))
        {
            System.out.println(s);
        }
        else if ("err".equals(_destination))
        {
            System.err.println(s);
        }
        else
        {
            Log log = _log;

            if (log == null)
            {
                log = LogFactory.getLog(_category);
            }
            
            if ("trace".equals(_destination))
            {
                log.trace(s);
            }
            else if ("debug".equals(_destination))
            {
                log.debug(s);
            }
            else if ("info".equals(_destination))
            {
                log.info(s);
            }
            else if ("warn".equals(_destination))
            {
                log.warn(s);
            }
            else if ("error".equals(_destination))
            {
                log.error(s);
            }
            else if ("fatal".equals(_destination))
            {
                log.fatal(s);
            }
        }
    }
}
