/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util;

import com.micronova.jsp.tag.YuzuTag;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

/** A Map to access scoped PageContext attributes (backed by PageContext) */

public class PageContextMap extends NestedMap
{
    protected PageContext _pageContext;
    protected int _scope;

    public PageContextMap(PageContext pageContext, int scope, List list)
    {
        super();

        _pageContext = pageContext;
        _scope = scope;

        Enumeration enumeration = null;

        if ((list != null) && (!list.isEmpty()))
        {
            enumeration = Collections.enumeration(list);
        }

        if (enumeration == null)
        {
            // session scope is handled separately

            if (scope == PageContext.SESSION_SCOPE)
            {
                enumeration = ((HttpServletRequest)_pageContext.getRequest()).getSession().getAttributeNames();
            }
            else
            {
                enumeration = _pageContext.getAttributeNamesInScope(_scope);
            }
        }

        while (enumeration.hasMoreElements())
        {
            String name = enumeration.nextElement().toString();
            this.put(name, pageContext.getAttribute(name, _scope));
        }
    }

    public PageContextMap(PageContext pageContext, int scope)
    {
        this(pageContext, scope, null);
    }
    
    public PageContext getPageContext()
    {
        return _pageContext;
    }

    public int getScope()
    {
        return _scope;
    }

    public Object putObject(Object objectSource, Object key, Object value)
    {
        String keyString = key.toString();
        PageContext pageContext = _pageContext;

        YuzuTag.setScopedAttribute(pageContext, keyString, value, _scope);

        return super.putObject(objectSource, key, value);
    }
}
