/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util;

import java.io.Reader;
import java.io.Writer;

/** I/O pipe (Runnable copying InputStream to OutputStream) */

public class Pipe implements Runnable
{
    protected Writer _writer;
    protected Reader _reader;
    protected char[] _buffer;
    protected boolean _doesCloseWriter;
    protected boolean _doesCloseReader;
   
    public static final int BUFFERSIZE = 1024;

    public Pipe(Reader reader, boolean doesCloseReader, Writer writer, boolean doesCloseWriter, char[] buffer)
    {
        super();

        _reader = reader;
        _doesCloseReader = doesCloseReader;

        _writer = writer;
        _doesCloseWriter = doesCloseWriter;

        if (buffer == null)
        {
            buffer = new char[BUFFERSIZE];
        }

        _buffer = buffer;
    }

    public Pipe(Reader reader, boolean doesCloseReader, Writer writer, boolean doesCloseWriter)
    {
        this(reader, doesCloseReader, writer, doesCloseWriter, null);
    }

    public void run()
    {
        Reader reader = _reader;
        Writer writer =  _writer;

        try
        {
            char[] buffer = _buffer;
            int bufferLength = buffer.length;
        
            for (;;)
            {
                int readLength = reader.read(buffer, 0, bufferLength);

                if (readLength > 0)
                {
                    writer.write(buffer, 0, readLength);

                    writer.flush();
                }
                else
                {
                    break;
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (_doesCloseWriter)
            {
                try
                {
                    writer.close();
                }
                catch (Exception ee)
                {
                    ee.printStackTrace();
                }
            }
            
            if (_doesCloseReader)
            {
                try
                {
                    reader.close();
                }
                catch (Exception ee)
                {
                    ee.printStackTrace();
                }
            }
        }
    }
}
