/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util.codec;

import com.micronova.jsp.tag.EL;
import com.micronova.jsp.tag.ELComparator;
import com.micronova.jsp.tag.ELIterator;
import com.micronova.jsp.tag.ELMap;
import com.micronova.jsp.tag.MessageMap;
import com.micronova.jsp.tag.YuzuTag;
import com.micronova.util.BeanUtil;
import com.micronova.util.LogWriter;
import com.micronova.util.NestedMap;
import com.micronova.util.PageContextMap;
import com.micronova.util.ServletUtil;
import com.micronova.util.Template;
import com.micronova.util.TypeUtil;
import com.micronova.util.servlet.DispatchHttpServletRequest;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.taglibs.standard.tag.common.core.Util;

/** JSP codecs */

public class CodecJSP extends Codec
{
    /** evaluates EL or map of EL */

    public static Object eval(PageContext pageContext, Object object, Pattern pattern, Object environment, String evalCodec, String keyCodec, boolean recursive, boolean allowGroup) throws Exception
    {
        if (object != null)
        {
            Object valueVarSaved = null;
            boolean isValueVarChanged = false;

            try
            {
                EL el = new EL(pageContext, evalCodec, keyCodec);
                
                valueVarSaved = YuzuTag.getValueVar(pageContext);
                
                if (environment != null)
                {
                    YuzuTag.setValueVar(pageContext, new NestedMap(environment));
                }

                isValueVarChanged = true;

                if (object instanceof NestedMap)
                {
                    NestedMap params = (NestedMap)((NestedMap)object).get("__param");
                    NestedMap evaluatedMap = new NestedMap();
                    
                    Iterator iterator = params.entrySet().iterator();

                    while (iterator.hasNext())
                    {
                        Map.Entry entry = (Map.Entry)iterator.next();

                        String key = Template.render(entry.getKey().toString(), pattern, 1, el, null, recursive, allowGroup).toString();

                        Object value = entry.getValue();

                        if (value instanceof String)
                        {
                            value = Template.render(value.toString(), pattern, 1, el, null, recursive, allowGroup);
                        }

                        evaluatedMap.setElement(key, value);
                    }
                    
                    object = evaluatedMap;
                }
                else
                {
                    object = Template.render(object.toString(), pattern, 1, el, null, recursive, allowGroup);
                }
            }
            catch (Exception e)
            {
                throw e;
            }
            finally
            {
                if (isValueVarChanged)
                {
                    YuzuTag.setValueVar(pageContext, valueVarSaved);
                }
            }
        }

        return object;
    }

    /** keys for eval control map */

    public final static String PATTERN = "pattern";
    public final static String RECURSIVE = "recursive";
    public final static String ALLOWGROUP = "allowGroup";
    public final static String ENVIRONMENT = "environment";
    public final static String EVALCODEC = "evalCodec";
    public final static String KEYCODEC = "keyCodec";

    /** evaluates EL or map of EL using control map */

    public static Object eval(Object context, Object object, Object control) throws Exception
    {
        if (object != null)
        {
            Object valueVarSaved = null;
            boolean isValueVarChanged = false;

            PageContext pageContext = (PageContext)context;

            NestedMap controlMap = TypeUtil.isNestedMap(control);

            Pattern pattern = EL.getPattern(controlMap.get(PATTERN));
            
            if (pattern == null)
            {
                pattern = EL.defaultPatternEval;
            }
            
            boolean recursive = (TypeUtil.isBoolean(controlMap.get(RECURSIVE, Boolean.FALSE))).booleanValue();
            boolean allowGroup = (TypeUtil.isBoolean(controlMap.get(ALLOWGROUP, Boolean.FALSE))).booleanValue();
            Object environment = controlMap.get(ENVIRONMENT);
            String evalCodec = controlMap.getString(EVALCODEC);
            String keyCodec = controlMap.getString(KEYCODEC);
            
            object = eval(pageContext, object, pattern, environment, evalCodec, keyCodec, recursive, allowGroup);
        }

        return object;
    }

    /** evaluate using default arguments */

    public static Object eval(Object context, Object object) throws Exception
    {
        return eval(context, object, null);
    }

    /** encodes URL */

    public static Object encodeURL(Object context, Object object) throws Exception
    {
        if (object != null)
        {
            object = ((HttpServletResponse)((PageContext)context).getResponse()).encodeURL(object.toString());
        }

        return object;
    }

    /** encodes redirect URL */

    public static Object encodeRedirectURL(Object context, Object object) throws Exception
    {
        if (object != null)
        {
            object = ((HttpServletResponse)((PageContext)context).getResponse()).encodeRedirectURL(object.toString());
        }

        return object;
    }

    /** returns real path */

    public static Object getRealPath(Object context, Object object)
    {
        if (object != null)
        {
            object = (((PageContext)context).getServletContext()).getRealPath(object.toString());
        }

        return object;
    }

    /** returns resource URL */

    public static Object getResource(Object context, Object object) throws Exception
    {
        if (object != null)
        {
            object = (((PageContext)context).getServletContext()).getResource(object.toString());
        }

        return object;
    }

    /** returns mimetype */

    public static Object getMimeType(Object context, Object object)
    {
        if (object != null)
        {
            object = (((PageContext)context).getServletContext()).getMimeType(object.toString());
        }

        return object;
    }

    /** creates a dynamic map */

    public static Object dynamicMap(Object context, Object object) throws Exception
    {
        return new ELMap((PageContext)context, object);
    }

    /** creates a dynamic iterator */

    public static Object dynamicIterator(Object context, Object object) throws Exception
    {
        return new ELIterator((PageContext)context, object);
    }

    /** applies given codec */

    public static Object applyCodec(Object context, Object object, Object codec) throws Exception
    {
        return EL.applyCodec((PageContext)context, EL.replaceEvalEscape(codec.toString()), object);
    }

    /** applies filter */

    public static Object applyFilter(Object context, Object object, Object filter) throws Exception
    {
        NestedMap filterMap = new NestedMap(filter);

        String include = EL.replaceEvalEscape(filterMap.getString(EL.INCLUDE));
        String breakExpression = EL.replaceEvalEscape(filterMap.getString(EL.BREAK));
        String apply = EL.replaceEvalEscape(filterMap.getString(EL.APPLY));
        String applyCodec = EL.replaceEvalEscape(filterMap.getString(EL.APPLYCODEC));
        
        return EL.applyFilter((PageContext)context, object, include, breakExpression, apply, applyCodec);
    }

    /** converts relative path (not string with a "/") to absolute, if necessary */

    public static Object getPath(Object context, Object object) throws Exception
    {
        String path = null;

        if (object != null)
        {
            PageContext pageContext = (PageContext)context;
            
            path = object.toString();

            if (!path.startsWith("/"))
            {
                path = ((HttpServletRequest)pageContext.getRequest()).getServletPath().replaceFirst("/[^/]*$", "") + "/" + path;
            }
        }
            
        return path;
    }

    public static final String RETURNPROPERTY = com.micronova.jsp.tag.CallTag.RETURNPROPERTY;
    public static final String VALUEPROPERTY = com.micronova.jsp.tag.CallTag.VALUEPROPERTY;
    public static final String BODYPROPERTY = com.micronova.jsp.tag.CallTag.BODYPROPERTY;

    /** calls a JSP */

    public static Object call(Object context, Object object, Object control) throws Exception
    {
        if (object != null)
        {
            PageContext pageContext = (PageContext)context;
            NestedMap controlMap = TypeUtil.isNestedMap(control);

            String path = (String)getPath(context, controlMap.get("path"));
            String contextPath = controlMap.getString("contextPath");
            boolean doesForward = TypeUtil.isTrue(controlMap.getString("doesForward", "false"));

            NestedMap map = TypeUtil.isNestedMap(object);

            object = ServletUtil.call(pageContext, path, contextPath, map, doesForward);
        }

        return object;
    }

    public static Object getLogWriter(Object context, Object control) throws Exception
    {
        PageContext pageContext = (PageContext)context;

        NestedMap controlMap = TypeUtil.isNestedMap(control);

        String destination = controlMap.getString("destination");

        if (destination == null)
        {
            destination = (String)YuzuTag.getConfiguration(pageContext, "com.micronova.jsp.tag.LogTag.destination", "log");
        }

        String category = controlMap.getString("category");

        if (category == null)
        {
            category = (String)YuzuTag.getConfiguration(pageContext, "com.micronova.jsp.tag.LogTag.category", "");
        }

        Log log = (Log)controlMap.get("log");

        if (log == null)
        {
            log = (Log)YuzuTag.getConfiguration(pageContext, "com.micronova.jsp.tag.LogTag.log", null);
        }

        return new LogWriter(pageContext, destination, category, log);
    }

    /** simple log facility */

    public static Object log(Object context, Object object, Object control) throws Exception
    {
        if (object != null)
        {
            LogWriter logWriter = (LogWriter)getLogWriter(context, control);
            logWriter.write(object.toString());
        }

        return object;
    }

    public static Object log(Object context, Object object) throws Exception
    {
        return log(context, object, null);
    }

    /** sorts a collection */

    public static Object sort(Object context, Object object, Object comparatorSpec) throws Exception
    {
        if (object != null)
        {
            List list = TypeUtil.isList(object);
            
            if (list != null)
            {
                if (comparatorSpec != null)
                {
                    ELComparator comparator = new ELComparator((PageContext)context, comparatorSpec);

                    Collections.sort(list, comparator);
                }
                else
                {
                    Collections.sort(list);
                }
            }
            
            object = list;
        }

        return object;
    }

    public static Object sort(Object context, Object object) throws Exception
    {
        return sort(context, object, null);
    }

    /** finds max in a collection using given comparator */

    public static Object max(Object context, Object object, Object comparatorSpec) throws Exception
    {
        if (object != null)
        {
            if (comparatorSpec != null)
            {
                ELComparator comparator = new ELComparator((PageContext)context, comparatorSpec);

                object = Collections.max((Collection)object, comparator);
            }
            else
            {
                object = Collections.max((Collection)object);
            }
        }

        return object;
    }

    public static Object max(Object context, Object object) throws Exception
    {
        return max(context, object, null);
    }

    /** finds min in a collection using given comparator */

    public static Object min(Object context, Object object, Object comparatorSpec) throws Exception
    {
        if (object != null)
        {
            if (comparatorSpec != null)
            {
                ELComparator comparator = new ELComparator((PageContext)context, comparatorSpec);

                object = Collections.min((Collection)object, comparator);
            }
            else
            {
                object = Collections.min((Collection)object);
            }
        }

        return object;
    }

    public static Object min(Object context, Object object) throws Exception
    {
        return min(context, object, null);
    }

    /** perform binary search on given object as a List using given comparatorSpec */

    public static Object binarySearch(Object context, Object object, Object key, Object comparatorSpec) throws Exception
    {
        List list = TypeUtil.isList(object);
        
        int index = -1;

        if (list != null)
        {
            if (comparatorSpec != null)
            {
                ELComparator comparator = new ELComparator((PageContext)context, comparatorSpec);
                
                index = Collections.binarySearch(list, key, comparator);
            }
            else
            {
                index = Collections.binarySearch(list, key);
            }
        }

        return new Integer(index);
    }

    public static Object binarySearch(Object context, Object object, Object key) throws Exception
    {
        return binarySearch(context, object, key, null);
    }

    /** create message map */

    public static Object getMessageMap(Object context, Object object) throws Exception
    {
        MessageMap messageMap = new MessageMap((PageContext)context);

        if (object != null)
        {
            BeanUtil.fill(messageMap, TypeUtil.isNestedMap(object));
        }

        return messageMap;
    }

    public static Object getMessageMap(Object context) throws Exception
    {
        return getMessageMap(context, null);
    }

    /** create EL comparator */

    public static Object comparator(Object context, Object object) throws Exception
    {
        return new ELComparator((PageContext)context, TypeUtil.isNestedMap(object));
    }

    /** flushes stream */

    public static Object flush(Object context, Object object) throws Exception
    {
        Writer writer = ((PageContext)context).getOut();

        if (object instanceof Writer)
        {
            writer = (Writer)object;
        }
        
        writer.flush();

        return object;
    }

    public static Object flush(Object context) throws Exception
    {
        return flush(context, null);
    }

    /** clears output */

    public static Object clear(Object context, Object object) throws Exception
    {
        JspWriter jspWriter = ((PageContext)context).getOut();

        if (object instanceof JspWriter)
        {
            jspWriter = (JspWriter)object;
        }

        jspWriter.clear();

        return object;
    }

    public static Object clear(Object context) throws Exception
    {
        return clear(context, null);
    }

    /** returns session as PageContextMap for the session if there is a session, otherwise null */

    public static Object hasSession(Object context) throws Exception
    {
        PageContext pageContext = (PageContext)context;

        HttpSession session = ((HttpServletRequest)pageContext.getRequest()).getSession(false);

        if (session != null)
        {
            return new PageContextMap(pageContext, PageContext.SESSION_SCOPE);
        }
        else
        {
            return null;
        }
    }

    /** invalidates current session and creates a new one with same session attributes returned as HttpSessionMap, if any */

    public static Object newSession(Object context) throws Exception
    {
        PageContext pageContext = (PageContext)context;

        HttpServletRequest request = ((HttpServletRequest)pageContext.getRequest());

        HttpSession session = request.getSession(false);

        NestedMap currentSessionMap = null;

        if (session != null)
        {
            currentSessionMap = new PageContextMap(pageContext, PageContext.SESSION_SCOPE);
            session.invalidate();
        }

        session = request.getSession(true);

        NestedMap sessionMap = new PageContextMap(pageContext, PageContext.SESSION_SCOPE);

        if (currentSessionMap != null)
        {
            sessionMap.copyFromSource(currentSessionMap);
        }

        return sessionMap;
    }

    /** invalidates current session if any.  Returns null. */

    public static Object invalidateSession(Object context) throws Exception
    {
        HttpServletRequest request = ((HttpServletRequest)((PageContext)context).getRequest());

        HttpSession session = request.getSession(false);

        if (session != null)
        {
            session.invalidate();
        }

        return null;
    }

    /** returns a shallow copy of current call stack (reversed so that the first element is the closest ancestor) */

    public static Object getCallStack(Object context) throws Exception
    {
        PageContext pageContext = (PageContext)context;

        List list = new ArrayList(com.micronova.jsp.tag.YuzuTag.getCallStack(pageContext));

        Collections.reverse(list);

        return list;
    }

    /** returns attribute map for given scope */

    public static Object getAttributes(Object context, Object vars, Object scopeName) throws Exception
    {
        int scope = PageContext.PAGE_SCOPE;

        if (scopeName != null)
        {
            scope = Util.getScope(scopeName.toString());
        }

        List list = TypeUtil.isStringList(vars, ',', '\\');

        return new PageContextMap((PageContext)context, scope, list);
    }

    public static Object getAttributes(Object context, Object vars) throws Exception
    {
        return getAttributes(context, vars, null);
    }

    public static Object getAttributes(Object context) throws Exception
    {
        return getAttributes(context, null);
    }

    /** extract map key/values into given scope and returns source */

    public static Object putAttributes(Object context, Object source, Object scopeName) throws Exception
    {
        PageContext pageContext = (PageContext)context;

        NestedMap sourceMap = TypeUtil.isNestedMap(source);

        Iterator iterator = sourceMap.entrySet().iterator();

        int scope = PageContext.PAGE_SCOPE;

        if (scopeName != null)
        {
            scope = Util.getScope(scopeName.toString());
        }

        while (iterator.hasNext())
        {
            Map.Entry entry = (Map.Entry)iterator.next();

            YuzuTag.setScopedAttribute(pageContext, entry.getKey().toString(), entry.getValue(), scope);
        }

        return source;
    }

    public static Object putAttributes(Object context, Object source) throws Exception
    {
        return putAttributes(context, source, null);
    }

    /** Saves current pageContext as rootPageContext in requestScope.  This becomes the root pageContext in the call stack (see getCallStack()).  If doesOverwrite (boolean) is true, then always replaces the value; otherwise only when the current value is null (default is false).  Returns the value before replacement.*/

    public static Object setRootPageContext(Object context, Object doesOverwrite) throws Exception
    {
        PageContext pageContext = (PageContext)context;

        Object currentRootPageContext = pageContext.getRequest().getAttribute(YuzuTag.ROOTPAGECONTEXT);

        if ((currentRootPageContext == null) || (TypeUtil.isTrue(doesOverwrite)))
        {
            pageContext.getRequest().setAttribute(YuzuTag.ROOTPAGECONTEXT, pageContext);
        }

        return currentRootPageContext;
    }

    public static Object setRootPageContext(Object context) throws Exception
    {
        return setRootPageContext(context, Boolean.FALSE);
    }

    /** retrieves pageScope variables as AttributeMap for entry in the call stack at given index (default 0).  This can be used to get/set particular pageScope variables. */

    public static Object getPageScope(Object context, Object indexSpec) throws Exception
    {
        List callStack = (List)getCallStack(context);
        int index = TypeUtil.isInteger(indexSpec).intValue();
        PageContext targetPageContext = (PageContext)TypeUtil.isNestedMap(callStack.get(index)).get(YuzuTag.PAGECONTEXT);
        return getAttributes(targetPageContext);
    }

    public static Object getPageScope(Object context) throws Exception
    {
        return getPageScope(context, new Integer(0));
    }

    /** sets HttpResponse according to control specification (as in "m:response" tag).  Returns object as-is. */

    public static Object setResponse(Object context, Object object, Object control) throws Exception
    {
        ServletUtil.setResponse((HttpServletResponse)((PageContext)context).getResponse(), TypeUtil.isNestedMap(control));
        return object;
    }

    /** returns TRUE if called from another JSP */

    public static Object isCalled(Object context)
    {
        ServletRequest request = ((PageContext)context).getRequest();

        if (request instanceof DispatchHttpServletRequest)
        {
            return Boolean.TRUE;
        }
        else
        {
            return Boolean.FALSE;
        }
    }
}
