/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util.codec;

import com.micronova.util.NestedMap;
import com.micronova.util.TypeUtil;
import java.util.Iterator;
import java.util.Map;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Script;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class CodecJavascript extends Codec
{
    /** evaluates/compiles Javascript expression using rhino */

    public static final Object eval(Object object, Object environment) throws Exception
    {
        if (object != null)
        {
            NestedMap envMap = TypeUtil.isNestedMap(environment);
            Context cx = Context.enter();

            try
            {
                Scriptable scope = cx.initStandardObjects();

                if (envMap != null)
                {
                    Iterator iterator = envMap.entrySet().iterator();

                    while (iterator.hasNext())
                    {
                        Map.Entry entry = (Map.Entry)iterator.next();
                        ScriptableObject.putProperty(scope, entry.getKey().toString(), Context.javaToJS(entry.getValue(), scope));
                    }
                }

                if (object instanceof Script)
                {
                    object = ((Script)object).exec(cx, scope);
                }
                else
                {
                    object = cx.evaluateString(scope, object.toString(), "<jsp>", 1, null);
                }
                    
                object = cx.jsToJava(object, java.lang.Object.class);
            }
            catch (Exception e)
            {
                throw e;
            }
            finally
            {
                Context.exit();
            }
        }
        
        return object;
    }

    public static final Object eval(Object object) throws Exception
    {
        return eval(object, null);
    }

    /** compiles a script.  Returned object can be used as "value" in eval. */

    public static final Object compile(Object object) throws Exception
    {
        if (object != null)
        {
            Context cx = Context.enter();
            try
            {
                Scriptable scope = cx.initStandardObjects();
        
                object = cx.compileString(object.toString(), "<jsp>", 1, null);
            }
            catch (Exception e)
            {
                throw e;
            }
            finally
            {
                cx.exit();
            }
        }
        return object;
    }
}
