/* 

Copyright 2003-2008 MicroNova (R)
All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

    * Neither the name of MicroNova nor the names of its contributors
    may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

package com.micronova.util.codec;

import com.micronova.util.NestedMap;
import com.micronova.util.StringUtil;
import com.micronova.util.TypeUtil;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.Writer;
import java.util.List;
import java.util.regex.Pattern;

/** String codecs */

public class CodecString extends Codec
{
    /** compiles pattern if patternSpec is a String */

    protected static Pattern getPattern(Object patternSpec)
    {
        Pattern pattern = null;

        if (patternSpec instanceof Pattern)
        {
            pattern = (Pattern)patternSpec;
        }
        else if (patternSpec != null)
        {
            pattern = Pattern.compile(patternSpec.toString());
        }

        return pattern;
    }

    /** replace all substrings matching pattern with replacement */

    public static final Object replaceAll(Object object, Object pattern, Object replacement)
    {
        if (object != null)
        {
            object = object.toString().replaceAll(pattern.toString(), replacement.toString());
        }

        return object;
    }

    public static final Object replaceAll(Object object, Object pattern)
    {
        return replaceAll(object, pattern, "");
    }

    /** replace first substring matching with pattern with replacement */

    public static final Object replaceFirst(Object object, Object pattern, Object replacement)
    {
        if (object != null)
        {
            object = object.toString().replaceFirst(pattern.toString(), replacement.toString());
        }

        return object;
    }

    public static final Object replaceFirst(Object object, Object pattern)
    {
        return replaceFirst(object, pattern, "");
    }

    /** return list of matching groups */

    public static final Object matchingGroups(Object object, Object patternSpec)
    {
        if (object != null)
        {
            object = StringUtil.matchingGroups(object.toString(), TypeUtil.isPattern(patternSpec));
        }

        return object;
    }

    /** 

    // splits string at pattern into List of strings 

    public static final Object split(Object object, Object pattern)
    {
        if (object != null)
        {
            object = TypeUtil.isList(object.toString().split(pattern.toString()));
        }

        return object;
    }

    // splits string at "," 

    public static final Object split(Object object)
    {
        return split(object, ",");
    }

    */

    /** trims string */

    public static final Object trim(Object object)
    {
        if (object != null)
        {
            object = object.toString().trim();
        }

        return object;
    }

    /** converts to upper case */

    public static final Object toUpperCase(Object object)
    {
        if (object != null)
        {
            object = object.toString().toUpperCase();
        }

        return object;
    }

    /** converts to lower case */

    public static final Object toLowerCase(Object object)
    {
        if (object != null)
        {
            object = object.toString().toLowerCase();
        }

        return object;
    }

    /** capitalize */

    public static final Object capitalize(Object object)
    {
        if (object != null)
        {
            object = StringUtil.capitalize(object.toString());
        }

        return object;
    }

    /** reverse */

    public static final Object reverse(Object object)
    {
        if (object != null)
        {
            object = StringUtil.reverse(object.toString());
        }

        return object;
    }

    /** multiply */

    public static final Object multiply(Object object, Object countSpec)
    {
        if (object != null)
        {
            Integer countInteger = TypeUtil.isInteger(countSpec);

            int count = (countInteger != null) ? countInteger.intValue() : 0;

            object = StringUtil.multiply(object.toString(), count);
        }

        return object;
    }

    /** append; if object is an instance of Writer, then simply writes to it. */

    public static final Object append(Object object, Object appended, Object condition) throws Exception
    {
        if (TypeUtil.isTrue(condition))
        {
            if (object == null)
            {
                object = appended;
            }
            else
            {
                if (appended != null)
                {
                    String appendedString = appended.toString();

                    if (object instanceof Writer)
                    {
                        ((Writer)object).write(appendedString);
                    }
                    else if (object instanceof OutputStream)
                    {
                        ((OutputStream)object).write(appendedString.getBytes("ISO-8859-1"));
                    }
                    else
                    {
                        object = object.toString() + appendedString;
                    }
                }
            }
        }

        return object;
    }

    public static final Object append(Object object, Object appended) throws Exception
    {
        return append(object, appended, Boolean.TRUE);
    }

    /** prepend; if prepended is an instance of Writer, then simply writes to it. */

    public static final Object prepend(Object object, Object prepended, Object condition) throws Exception
    {
        if (TypeUtil.isTrue(condition))
        {
            if (object != null)
            {
                if (prepended != null)
                {
                    String objectString = object.toString();

                    if (prepended instanceof Writer)
                    {
                        ((Writer)prepended).write(objectString);
                    }
                    else if (prepended instanceof OutputStream)
                    {
                        ((OutputStream)prepended).write(objectString.getBytes("ISO-8859-1"));
                    }
                    else
                    {
                        object = prepended.toString() + objectString;
                    }
                }
            }
            else
            {
                object = prepended;
            }
        }
            
        return object;
    }

    public static final Object prepend(Object object, Object prepended) throws Exception
    {
        return prepend(object, prepended, Boolean.TRUE);
    }

    /** returns substring from start to end */

    public static final Object substring(Object object, Object start, Object end)
    {
        if (object != null)
        {
            String string = object.toString();

            int length = string.length();

            int startIndex = TypeUtil.isInteger(start).intValue();
            int endIndex = TypeUtil.isInteger(end).intValue();

            object = StringUtil.substring(string, startIndex, endIndex);
        }

        return object;
    }

    /** returns substring starting at start */

    public static final Object substring(Object object, Object start)
    {
        if (object != null)
        {
            String string = object.toString();

            int startIndex = TypeUtil.isInteger(start).intValue();

            object = StringUtil.substring(string, startIndex);
        }

        return object;
    }

    /** finds index of given substring */

    public static final Object indexOf(Object object, Object sub)
    {
        if ((object != null) && (sub != null))
        {
            object = new Integer(object.toString().indexOf(sub.toString()));
        }

        return object;
    }

    /** finds last index of given substring */

    public static final Object lastIndexOf(Object object, Object sub)
    {
        if ((object != null) && (sub != null))
        {
            object = new Integer(object.toString().lastIndexOf(sub.toString()));
        }

        return object;
    }

    /** returns string length */

    public static final Object length(Object object)
    {
        if (object != null)
        {
            return new Integer(object.toString().length());
        }
        
        return object;
    }

    /** converts String into Character list */

    public static final Object toCharacterList(Object object)
    {
        if (object != null)
        {
            return TypeUtil.isList(object.toString().toCharArray());
        }

        return object;
    }

    /** converts List of Characters to String */

    public static final Object fromCharacterList(Object object)
    {
        if (object != null)
        {
            List list = (List)TypeUtil.isList(object);

            int length = list.size();
            
            char[] array = new char[length];

            for (int i = length; --i >= 0;)
            {
                array[i] = ((Character)(list.get(i))).charValue();
            }

            object = new String(array);
        }

        return object;
    }


    /** split by patterns */

    public static final Object split(Object object, Object patternSpec, Object groupSpec)
    {
        if (object != null)
        {
            Pattern pattern = getPattern(patternSpec);

            int groupIndex = 0;

            Integer groupIndexInteger = TypeUtil.isInteger(groupSpec);

            if (groupIndexInteger != null)
            {
                groupIndex = groupIndexInteger.intValue();
            }

            object = StringUtil.split(object.toString(), pattern, groupIndex);
        }

        return object;
    }

    /** split by pattern */

    public static final Object split(Object object, Object patternSpec)
    {
        return split(object, patternSpec, null);
    }

    /** split by pattern */

    public static final Object split(Object object)
    {
        return split(object, ",");
    }

    /** list matching patterns */

    public static final Object matchAll(Object object, Object patternSpec, Object groupSpec)
    {
        if (object != null)
        {
            Pattern pattern = getPattern(patternSpec);

            int groupIndex = 0;

            Integer groupIndexInteger = TypeUtil.isInteger(groupSpec);

            if (groupIndexInteger != null)
            {
                groupIndex = groupIndexInteger.intValue();
            }

            object = StringUtil.matchAll(object.toString(), pattern, groupIndex);
        }

        return object;
    }

    /** list matching patterns */

    public static final Object matchAll(Object object, Object patternSpec)
    {
        return matchAll(object, patternSpec, null);
    }

    /** decompose by pattern */

    public static final Object decompose(Object object, Object patternSpec, Object groupSpec)
    {
        if (object != null)
        {
            Pattern pattern = getPattern(patternSpec);

            int groupIndex = 0;

            Integer groupIndexInteger = TypeUtil.isInteger(groupSpec);

            if (groupIndexInteger != null)
            {
                groupIndex = groupIndexInteger.intValue();
            }

            object = StringUtil.decompose(object.toString(), pattern, groupIndex);
        }

        return object;
    }

    /** decompose by pattern */

    public static final Object decompose(Object object, Object patternSpec)
    {
        return decompose(object, patternSpec, null);
    }


    /** returns the matching group, or null */

    public static final Object match(Object object, Object patternSpec, Object groupSpec)
    {
        if (object != null)
        {
            Pattern pattern = getPattern(patternSpec);

            int groupIndex = 0;

            Integer groupIndexInteger = TypeUtil.isInteger(groupSpec);

            if (groupIndexInteger != null)
            {
                groupIndex = groupIndexInteger.intValue();
            }

            object = StringUtil.match(object.toString(), pattern, groupIndex);
        }

        return object;
    }

    /** match the first one or null */

    public static final Object match(Object object, Object patternSpec)
    {
        return match(object, patternSpec, null);
    }

    /** joins elements of sequence object using glue */

    public static final Object join(Object object, Object glue)
    {
        if (object != null)
        {
            List list = TypeUtil.isList(object);

            if ((list != null) && (glue != null))
            {
                object = StringUtil.join(list, glue.toString());
            }
            else
            {
                object = null;
            }
        }
        
        return object;
    }
    
    /** join elements of sequence object using "," */

    public static final Object join(Object object)
    {
        return join(object, ",");
    }

    /** converts to byte array with specific encoding */

    public static final Object toByteArray(Object object, Object encoding) throws Exception
    {
        if (object != null)
        {
            String string = object.toString();

            if (encoding != null)
            {
                object = string.getBytes(encoding.toString());
            }
            else
            {
                object = string.getBytes();
            }
        }

        return object;
    }

    /** converts from byte array with specific encoding */

    public static final Object fromByteArray(Object object, Object encoding) throws Exception
    {
        if (object != null)
        {
            byte[] byteArray = (byte[])object;
            
            if (encoding != null)
            {
                object = new String(byteArray, encoding.toString());
            }
            else
            {
                object = new String(byteArray);
            }
        }
        
        return object;
    }

    /** word counts */

    public static final Object countWords(Object object, Object pattern)
    {
        if (object != null)
        {
            object = StringUtil.countWords(object.toString(), getPattern(pattern.toString()));
        }

        return object;
    }

    /** word counts using default splitting pattern ([ ]+) */
    
    public static final Object countWords(Object object)
    {
        return countWords(object, "[ ]+");
    }

    /** replaces characters */

    public static final Object replaceCharacters(Object object, Object from, Object to)
    {
        if (object != null)
        {
            object = StringUtil.replaceCharacters(object.toString(), from.toString().toCharArray(), to.toString().toCharArray());
        }

        return object;
    }

    /** split a string as CSV into a list of lists of strings.  "spec" optionally specifies the following: "separator" (default ","), "quote (default "\n"), "newline" (default "\n"). "spec" values can be backslash-encoded. */

    public static final Object splitCSV(Object object, Object spec) throws Exception
    {
        if (object != null)
        {
            NestedMap specMap = TypeUtil.isNestedMap(spec);
            String separator = StringUtil.decodeBackslash(specMap.getString("separator", ","));
            String quote = StringUtil.decodeBackslash(specMap.getString("quote", "\""));
            String newline = StringUtil.decodeBackslash(specMap.getString("newline", "\n"));

            object = StringUtil.splitCSV(new StringReader(object.toString()), separator.charAt(0), quote.charAt(0), newline.charAt(0));
        }

        return object;
    }

    public static final Object splitCSV(Object object) throws Exception
    {
        return splitCSV(object, null);
    }

    /** escapes unicode characters in given range (default all).  "controlSpec" specifies a map with "minCode" (default 128) and "maxCode" (default 65535). */

    public static final Object escapeUnicode(Object object, Object controlSpec)
    {
        if (object != null)
        {
            NestedMap controlMap = TypeUtil.isNestedMap(controlSpec);
            long minCode = controlMap.getLong("minCode", 128L);
            long maxCode = controlMap.getLong("maxCode", 65535L);
            object = StringUtil.escapeUnicode(object.toString(), minCode, maxCode);
        }

        return object;
    }

    public static final Object escapeUnicode(Object object)
    {
        return escapeUnicode(object, null);
    }

    /** simple replace without using regexp */

    public static final Object replace(Object object, Object substring, Object replacement)
    {
        if (object != null)
        {
            object = object.toString().replace((String)substring, (String)replacement);
        }
        
        return object;
    }

    public static final Object replace(Object object, Object substring)
    {
        return replace(object, substring, "");
    }

    /** converts to character array */

    public static final Object toCharacterArray(Object object)
    {
        if (object != null)
        {
            object = object.toString().toCharArray();
        }

        return object;
    }
}

